#ifndef _VORBISENC_H_
#define _VORBISENC_H_

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <vorbis/vorbisenc.h>
#include <shout/shout.h>

typedef int (DownSample)(int*, short*);

class Stream {

 public:
  Stream ();
  void Init();
  int Connect();
  void Send();
  int get_port() {return conn.port;}
  char * get_ip() {return conn.ip;}
  char * get_mount() {return conn.mount;}
  char * get_pass() {return conn.password;}
  void set_port(int i);
  void set_ip(char * c);
  void set_mount(char * c);
  void set_pass(char * c);
  Stream *next_stream;
  shout_conn_t conn;
};



class VorbisEncode {
 public:
  VorbisEncode();
  void Encode(char *readbuffer, long bytes, int channels);
  void Init();
  bool OpenOutfile( const char *filename);
  void CloseOutfile();
  void WriteOutfile();
  void CleanUp();
  Stream * AddStream();

  Stream * get_first_stream() {return first_stream;}
  Stream * get_current_stream() {return current_stream;}
  int get_bitrate() {return bitrate;}
  int get_samplerate() {return samplerate;}
  int get_channels() {return channels;}
  char * get_savefile() {return savefile;}
  bool do_save() {return save;}
  bool do_encode();

  void set_save(bool i);
  void set_bitrate(int i);
  void set_channels(int i);
  void set_samplerate(int i);
  void set_savefile(char * c);
  
  DownSample *downsamp;


 private:
  ogg_stream_state os; /* take physical pages, weld into a logical
			  stream of packets */
  ogg_page         og; /* one Ogg bitstream page.  Vorbis packets are inside */
  ogg_packet       op; /* one raw packet of data for decode */
  vorbis_info      vi; /* struct that stores all the static vorbis bitstream
			  settings */
  vorbis_comment   vc; /* struct that stores all the user comments */
  vorbis_dsp_state vd; /* central working state for the packet->PCM decoder */
  vorbis_block     vb; /* local working space for packet->PCM decode */

  int eos;
  FILE *of;
  Stream *first_stream;
  Stream *current_stream;
  bool save;
  int channels;
  int bitrate;
  int samplerate;
  char *savefile;

};
#endif
