#include "vorbisenc.h"
#include "generic.h"
#include <string.h>

VorbisEncode::VorbisEncode()
{
  eos = 0;
  first_stream = NULL;
  save=false;
  bitrate=0;
  samplerate=44100;
  channels=2;
  savefile=NULL;
}


void VorbisEncode::Init( )
{
  Stream *ptr;

  /* choose an encoding mode */
  vorbis_info_init(&vi);
  //vi.channels = channels;
  //vi.rate = samplerate;  
  //vi.bitrate_nominal = bitrate;
  
  /*if ( vorbis_encode_init(&vi,channels, samplerate, -1, bitrate, -1) )
    {
      fprintf(stderr, "Mode initialisation failed: invalid parameters for quality\n");
      vorbis_info_clear(&vi);
      exit(1);
    }
  */
  if ( vorbis_encode_init_vbr(&vi, channels, samplerate,.001) )
    {
      fprintf(stderr, "Mode initialisation failed: invalid parameters for quality\n");
      vorbis_info_clear(&vi);
      exit(1);
    }
  
  fprintf(stderr, "Init %d \n", samplerate);      

  /* add a comment */
  vorbis_comment_init(&vc);
  vorbis_comment_add(&vc,"title=streamed with Oggment");
  
  /* set up the analysis state and auxiliary encoding storage */
  vorbis_analysis_init(&vd,&vi);
  

  vorbis_block_init(&vd,&vb);
 
  srand(time(NULL));
  ogg_stream_init(&os,rand());
  
  ogg_packet header;
  ogg_packet header_comm;
  ogg_packet header_code;

  vorbis_analysis_headerout(&vd,&vc,&header,&header_comm,&header_code);
  ogg_stream_packetin(&os,&header); /* automatically placed in its own page */
  ogg_stream_packetin(&os,&header_comm);
  ogg_stream_packetin(&os,&header_code);


  /* Connect to servers */
  ptr = first_stream;
  if (ptr == NULL) {
    fprintf(stderr, "VorbisEncode::Init - no streams\n");
  }
  while(ptr != NULL) {
    ptr->Connect();
    ptr = ptr->next_stream;
  }

}

void VorbisEncode::Encode(char *readbuffer, long bytes, int ch)
{
  long i; int j; int tmp;
  Stream *ptr;
  
  /* data to encode */
  float **buffer=vorbis_analysis_buffer(&vd, bytes);  //READ);
  
  /* expose the buffer to submit data */
  /* uninterleave samples */
  
  for(i = 0; i < bytes/(ch*2); i++)
    {
      for(j=0; j < ch; j++)
	{
	  buffer[j][i] = ((readbuffer[i*2*ch + 2*j + 1]<<8) |
			  (readbuffer[i*2*ch + 2*j] & 0xff))/32768.0f;
	}
    }
  
  /* tell the library how much we actually submitted */
  vorbis_analysis_wrote(&vd,i);
    
  /* vorbis does some data preanalysis, then divvies up blocks for
     more involved (potentially parallel) processing.  Get a single
     block for encoding now */
  while(vorbis_analysis_blockout(&vd,&vb)==1){

    /* analysis */
    vorbis_analysis(&vb,&op);
    
    /* weld the packet into the bitstream */
    ogg_stream_packetin(&os,&op);

    while (!eos) {
      int result=ogg_stream_pageout(&os,&og);
      if(!result) {
	break;
      }
      if (of != NULL && save) {
	fwrite(og.header,1,og.header_len, of);
	fwrite(og.body,1,og.body_len, of);
      }

      //printf("shout: Inside encode: ...\n");

      ptr = first_stream;
      while(ptr != NULL) {
	tmp = shout_send_data(&ptr->conn, og.header, og.header_len);
	if (!tmp) 
	  printf("shout: Send error: %i...\n", ptr->conn.error);
	tmp = shout_send_data(&ptr->conn, og.body, og.body_len);
	if (!tmp) 
	  printf("shout: Send error: %i...\n", ptr->conn.error);
	//shout_sleep(&conn);
	ptr = ptr->next_stream;
      }      
      if(ogg_page_eos(&og))  eos=1;
    }
  }
}

Stream * VorbisEncode::AddStream()
{

  fprintf(stderr, "adding a stream\n");
  Stream *ptr;

  if (first_stream == NULL) {
    first_stream = new Stream();
    return first_stream;
  }
  else {
    ptr = first_stream->next_stream;
    while(ptr != NULL)
      ptr = ptr->next_stream;
    ptr= new Stream();
    return ptr;
  }
}


bool VorbisEncode::OpenOutfile(const char *filename) {
  if ( (of = fopen(filename, "w")) == NULL) {
    fprintf(stderr, "couldn't open vorbis outfile for writing");
    return 0;
  }
  set_savefile( (char *)filename);
  return 1;
}

bool VorbisEncode::do_encode() {
  if (savefile != NULL)
    return true;
  if (first_stream != NULL)
    return true;
  return false;
}

void VorbisEncode::CloseOutfile() {
  fclose(of);
}

void VorbisEncode::WriteOutfile() {
    fwrite(og.header,1,og.header_len, of);
    fwrite(og.body,1,og.body_len, of);
}


void VorbisEncode::CleanUp()
{
  /* clean up and exit.  vorbis_info_clear() must be called last */ 
  vorbis_analysis_wrote(&vd,0);  //this was the breaker in the encode loop
  ogg_stream_clear(&os);
  vorbis_block_clear(&vb);
  vorbis_dsp_clear(&vd);
  vorbis_comment_clear(&vc);
  vorbis_info_clear(&vi);
  if (of != NULL) 
    fclose(of);
}

void VorbisEncode::set_bitrate(int i) {
  bitrate = i;
}
void VorbisEncode::set_samplerate(int i) {
  samplerate = i;
}
void VorbisEncode::set_channels(int i) {
  channels = i;
}
void VorbisEncode::set_savefile(char * c) {
  savefile = strdup(c);
}
void VorbisEncode::set_save(bool i) {
  save = i;
}


///////////////////////////////////////////////////////////

Stream::Stream( )
{
  next_stream = NULL;
  conn.ip = NULL;
  conn.mount = NULL;
  conn.port = 0;
  conn.password = NULL;
  shout_init_connection(&conn);
}


int Stream::Connect()
{
  if (shout_connect(&conn)) {
    printf("Connected to server...\n");
  } else { 
    printf("Stream::Connect - not connected to server...\n");
    return -1; }

  return 0;
}
void Stream::set_port(int i) {
  conn.port = i;
}
void Stream::set_ip(char * c) {
  conn.ip = strdup(c);
}
void Stream::set_mount(char * c) {
  conn.mount = strdup(c);
}
void Stream::set_pass(char * c) {
  conn.password = strdup(c);
}
/*
  void Stream::Send(VorbisEncode *ve) {
   int tmp;
   tmp = shout_send_data(&conn, ve->og.header, ve->og.header_len);
   if (!tmp) 
   printf("shout: Send error: %i...\n", conn.error);
   tmp = shout_send_data(&conn, ve->og.body, ve->og.body_len);
   if (!tmp) 
   printf("shout: Send error: %i...\n", conn.error);
   //shout_sleep(&conn);
   }
*/
